--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **    Project name: BinCnt                                              **
-- **    Module: bincnt (root)                                             **
-- **    Target device: CoolRunner II (XC2C256/VQ100) [ZL14PLD/ZL15PLD]    **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;
 
entity BinCnt is port ( Clk : in std_logic ;
                        Reset : in std_logic ;
                        Seg_A : out std_logic ;
                        Seg_B : out std_logic ;
                        Seg_C : out std_logic ;
                        Seg_D : out std_logic ;
                        Seg_E : out std_logic ;
                        Seg_F : out std_logic ;
                        Seg_G : out std_logic ;
                        Seg_K : out std_logic ;
                        DSel0 : out std_logic ;
                        DSel1 : out std_logic ;
                        DSel2 : out std_logic ;
                        DSel3 : out std_logic ) ;
end BinCnt ;

architecture Behavioral of BinCnt is

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot   : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  signal DClk : std_logic ;
  signal CntClk : std_logic ;
  signal Cnt : std_logic_vector ( 15 downto 0 ) ;
  signal FDiv : std_logic_vector ( 14 downto 0 ) ;

begin
  FDivInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if Clk'event and Clk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 6 ) ;

  CntClk <= FDiv ( FDiv'left ) ;

  process ( CntClk , Reset )
  begin
    if Reset = '0' then
      Cnt <= ( others => '0' ) ;
    else
      if CntClk'event and CntClk = '0' then
        Cnt <= Cnt + 1 ;
      end if ;
    end if ;
  end process ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Neg => '1' ,
                                            WA => '1' ,
                                            Tetr0 => Cnt ( 3 downto 0 ) ,
                                            Tetr1 => Cnt ( 7 downto 4 ) ,
                                            Tetr2 => Cnt ( 11 downto 8 ) ,
                                            Tetr3 => Cnt ( 15 downto 12 ) ,
                                            Dot => "1101" ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;

end Behavioral ;
