--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **             "Ukady programowalne CoolRunner II w praktyce"          **
-- **                                                                      **
-- **            Module: Mux4DigDisp (przydatny komponent)                 **
-- **            Target device: XILINX CPLD                                **
-- **            Tool versions: ISE WebPACK 9.2i                           **
-- **            Date: BIAYSTOK, stycze 2009                             **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity Mux4DigDisp is port ( DClk : in std_logic ;
                             Reset : in std_logic ;
                             Neg : in std_logic := '0' ;
                             WA : in std_logic := '0' ;
                             Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                             Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                             Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                             Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                             Dot : in std_logic_vector ( 3 downto 0 ) ;
                             Seg_A : out std_logic ;
                             Seg_B : out std_logic ;
                             Seg_C : out std_logic ;
                             Seg_D : out std_logic ;
                             Seg_E : out std_logic ;
                             Seg_F : out std_logic ;
                             Seg_G : out std_logic ;
                             Seg_K : out std_logic ;
                             DSel0 : out std_logic ;
                             DSel1 : out std_logic ;
                             DSel2 : out std_logic ;
                             DSel3 : out std_logic ) ;
end Mux4DigDisp ;

architecture Behavioral of Mux4DigDisp is

  component UHexConv port ( A : in std_logic ;
                            B : in std_logic ;
                            C : in std_logic ;
                            D : in std_logic ;
                            Neg : in std_logic ;
                            Seg_A : out std_logic ;
                            Seg_B : out std_logic ;
                            Seg_C : out std_logic ;
                            Seg_D : out std_logic ;
                            Seg_E : out std_logic ;
                            Seg_F : out std_logic ;
                            Seg_G : out std_logic ) ;
  end component ;

  signal SelCnt : std_logic_vector ( 1 downto 0 ) ;
  signal Tetr : std_logic_vector ( 3 downto 0 ) ;
  signal TSel0 : std_logic ;
  signal TSel1 : std_logic ;
  signal TSel2 : std_logic ;
  signal TSel3 : std_logic ;
  signal CurDot : std_logic ;

begin
  SelCntInstance : process ( DClk , Reset )
  begin
    if Reset = '0' then
      SelCnt <= "00" ;
    else
      if DClk'event and DClk = '0' then
        SelCnt <= SelCnt + 1 ;
      end if ;
    end if ;
  end process ;

  with SelCnt select
    Tetr <= Tetr3 when "11" ,
            Tetr2 when "10" ,
            Tetr1 when "01" ,
            Tetr0 when others ;

  with SelCnt select
    CurDot <= Dot ( 3 ) when "11" ,
              Dot ( 2 ) when "10" ,
              Dot ( 1 ) when "01" ,
              Dot ( 0 ) when others ;

  Seg_K <= Neg xor CurDot ;

  ConvInstance : UHexConv port map ( A => Tetr ( 0 ) ,
                                     B => Tetr ( 1 ) ,
                                     C => Tetr ( 2 ) ,
                                     D => Tetr ( 3 ) ,
                                     Neg => Neg ,
                                     Seg_A => Seg_A ,
                                     Seg_B => Seg_B ,
                                     Seg_C => Seg_C ,
                                     Seg_D => Seg_D ,
                                     Seg_E => Seg_E ,
                                     Seg_F => Seg_F ,
                                     Seg_G => Seg_G ) ;

  TSel0 <= '0' when SelCnt = "00" else '1' ;
  TSel1 <= '0' when SelCnt = "01" else '1' ;
  TSel2 <= '0' when SelCnt = "10" else '1' ;
  TSel3 <= '0' when SelCnt = "11" else '1' ;

  DSel0 <= TSel0 xor WA ;
  DSel1 <= TSel1 xor WA ;
  DSel2 <= TSel2 xor WA ;
  DSel3 <= TSel3 xor WA ;

end Behavioral ;
