--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **             "Ukady programowalne CoolRunner II w praktyce"          **
-- **                                                                      **
-- **    Project name: Clock                                               **
-- **    Module: Clock (root)                                              **
-- **    Target device: CoolRunner II (XC2C256/VQ100) [ZL14PLD/ZL15PLD]    **
-- **    Tool versions: ISE WebPACK 9.2i                                   **
-- **    Date: BIAYSTOK, stycze 2009                                     **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity Clock is port ( Clk : in std_logic ;
                       Reset : in std_logic ;
                       ModeKeyInp : in std_logic ;
                       SetupKeyInp : in std_logic ;
                       Seg_A : out std_logic ;
                       Seg_B : out std_logic ;
                       Seg_C : out std_logic ;
                       Seg_D : out std_logic ;
                       Seg_E : out std_logic ;
                       Seg_F : out std_logic ;
                       Seg_G : out std_logic ;
                       Seg_K : out std_logic ;
                       DSel0 : out std_logic ;
                       DSel1 : out std_logic ;
                       DSel2 : out std_logic ;
                       DSel3 : out std_logic ) ;
end Clock ;

architecture Behavioral of Clock is

  constant FDivCntSizeParam : integer := 3 ;
  constant DisplayNegParam : std_logic := '1' ;
  constant DisplayWAParam : std_logic := '1' ;

  component FDivider generic ( constant CntSize : integer := 8 ) ;
                     port ( Clk : in std_logic ;
                            Reset : in std_logic ;
                            FDiv : out std_logic ) ;
  end component ;

  component PushButton port ( KClk : in std_logic ;
                              Reset : in std_logic ;
                              KeyIn : in std_logic ;
                              Status : out std_logic ) ;
  end component ;

  component BCDConv port ( Binary : in std_logic_vector ( 5 downto 0 ) ;
                           BCD0 : out std_logic_vector ( 3 downto 0 ) ;
                           BCD1 : out std_logic_vector ( 3 downto 0 ) ) ;
  end component ;

  component Mux4DigDisp port ( DClk : in std_logic ;
                               Reset : in std_logic ;
                               Neg : in std_logic := '0' ;
                               WA : in std_logic := '0' ;
                               Tetr0 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr1 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr2 : in std_logic_vector ( 3 downto 0 ) ;
                               Tetr3 : in std_logic_vector ( 3 downto 0 ) ;
                               Dot   : in std_logic_vector ( 3 downto 0 ) ;
                               Seg_A : out std_logic ;
                               Seg_B : out std_logic ;
                               Seg_C : out std_logic ;
                               Seg_D : out std_logic ;
                               Seg_E : out std_logic ;
                               Seg_F : out std_logic ;
                               Seg_G : out std_logic ;
                               Seg_K : out std_logic ;
                               DSel0 : out std_logic ;
                               DSel1 : out std_logic ;
                               DSel2 : out std_logic ;
                               DSel3 : out std_logic ) ;
  end component ;

  type AutomatStateType is ( IdleState ,
                             SetupOptionState ,
                             DispModeOptionState ,
                             EndOptionState ) ;

  signal MachineState : AutomatStateType ;
  signal Sec : std_logic_vector ( 5 downto 0 ) ;
  signal Min : std_logic_vector ( 5 downto 0 ) ;
  signal Hrs : std_logic_vector ( 5 downto 0 ) ;
  signal SecEnable : std_logic ;
  signal MinEnable : std_logic ;
  signal HrsEnable : std_logic ;
  signal TMinEnable : std_logic ;
  signal THrsEnable : std_logic ;
  signal Sec1 : std_logic_vector ( 3 downto 0 ) ;
  signal Sec10 : std_logic_vector ( 3 downto 0 ) ;
  signal Min1 : std_logic_vector ( 3 downto 0 ) ;
  signal Min10 : std_logic_vector ( 3 downto 0 ) ;
  signal Hrs1 : std_logic_vector ( 3 downto 0 ) ;
  signal Hrs10 : std_logic_vector ( 3 downto 0 ) ;
  signal BCD0 : std_logic_vector ( 3 downto 0 ) ;
  signal BCD1 : std_logic_vector ( 3 downto 0 ) ;
  signal BCD2 : std_logic_vector ( 3 downto 0 ) ;
  signal BCD3 : std_logic_vector ( 3 downto 0 ) ;
  signal FDiv : std_logic_vector ( 4 downto 0 ) ;
  signal DotControl : std_logic_vector ( 3 downto 0 ) ;
  signal Context : std_logic_vector ( 1 downto 0 ) ;
  signal DClk : std_logic ;
  signal KClk : std_logic ;
  signal FClk : std_logic ;
  signal F1Hz : std_logic ;
  signal MinClk : std_logic ;
  signal HrsClk : std_logic ;
  signal ClrPresc : std_logic ;
  signal ClrSecs : std_logic ;
  signal DispMode : std_logic ;
  signal SetupMode : std_logic ;
  signal SetupKeyStatus : std_logic ;
  signal ModeKeyStatus : std_logic ;


begin
  FDivInstance : FDivider generic map ( CntSize => FDivCntSizeParam )
                          port map ( Clk => Clk ,
                                     Reset => Reset ,
                                     FDiv => FClk ) ;

  ClkGenInstance : process ( FClk , Reset )
  begin
    if Reset = '0' then
      FDiv <= ( others => '0' ) ;
    else
      if FClk'event and FClk = '0' then
        FDiv <= FDiv + 1 ;
      end if ;
    end if ;
  end process ;

  DClk <= FDiv ( 3 ) ;
  KClk <= FDiv ( 4 ) ;

  ModeKeyInstance : PushButton port map ( KClk => KClk ,
                                          Reset => Reset ,
                                          KeyIn => ModeKeyInp ,
                                          Status => ModeKeyStatus ) ;

  SetupKeyInstance : PushButton port map ( KClk => KClk ,
                                           Reset => Reset ,
                                           KeyIn => SetupKeyInp ,
                                           Status => SetupKeyStatus ) ;

  ClrSecs <= '0' when ( Context = "11" ) else '1' ;
  ClrPresc <= '0' when ( Reset = '0' or ClrSecs = '0' ) else '1' ;

  F1HzDivInstance : FDivider generic map ( CntSize => 15 )
                             port map ( Clk => Clk ,
                                        Reset => ClrPresc ,
                                        FDiv => F1Hz ) ;

  SetupMode <= '0' when ( Context = "00" ) else '1' ;

  SecEnable <= '1' when ( SetupMode = '0' ) else '0' ;

  TMinEnable <= '1' when ( SecEnable = '1' and Sec = "111011" ) else '0' ;
  THrsEnable <= '1' when ( TMinEnable = '1' and Min = "111011" ) else '0' ;

  with Context select
    MinEnable <= TMinEnable when "00" ,
                 '1' when "01" ,
                 '0' when others ;

  with Context select
    HrsEnable <= THrsEnable when "00" ,
                 '1' when "10" ,
                 '0' when others ;

  with Context select
    MinClk <= F1Hz when "00" ,
              ModeKeyStatus when "01" ,
              '0' when others ;

  with Context select
    HrsClk <= F1Hz when "00" ,
              ModeKeyStatus when "10" ,
              '0' when others ;

  process ( F1Hz , SecEnable , Reset , ClrSecs )
  begin
    if Reset = '0' or ClrSecs = '0' then
      Sec <= ( others => '0' ) ;
    else
      if SecEnable = '1' then
        if F1Hz'event and F1Hz = '0' then
          if Sec = "111011" then
            Sec <= ( others => '0' ) ;
          else
            Sec <= Sec + 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  process ( MinClk , MinEnable , Reset )
  begin
    if Reset = '0' then
      Min <= ( others => '0' ) ;
    else
      if MinEnable = '1' then
        if MinClk'event and MinClk = '0' then
          if Min = "111011" then
            Min <= ( others => '0' ) ;
          else
            Min <= Min + 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  process ( HrsClk , HrsEnable , Reset )
  begin
    if Reset = '0' then
      Hrs <= ( others => '0' ) ;
    else
      if HrsEnable = '1' then
        if HrsClk'event and HrsClk = '1' then
          if Hrs = "010111" then
            Hrs <= ( others => '0' ) ;
          else
            Hrs <= Hrs + 1 ;
          end if ;
        end if ;
      end if ;
    end if ;
  end process ;

  SCnvInstance : BCDConv port map ( Binary => Sec ,
                                    BCD0 => Sec1 ,
                                    BCD1 => Sec10 ) ;

  MCnvInstance : BCDConv port map ( Binary => Min ,
                                    BCD0 => Min1 ,
                                    BCD1 => Min10 ) ;

  HCnvInstance : BCDConv port map ( Binary => Hrs ,
                                    BCD0 => Hrs1 ,
                                    BCD1 => Hrs10 ) ;

  with DispMode select
    BCD0 <= Min1 when '1' ,
            Sec1 when others ;

  with DispMode select
    BCD1 <= Min10 when '1' ,
            Sec10 when others ;

  with DispMode select
    BCD2 <= Hrs1 when '1' ,
            Min1 when others ;

  with DispMode select
    BCD3 <= Hrs10 when '1' ,
            Min10 when others ;

  with Context select
    DotControl <= "1111" when "11" ,
                  "0011" when "10" ,
                  "1100" when "01" ,
                  '1' & F1Hz & "11" when others ;

  MuxDisplInstance : Mux4DigDisp port map ( DClk => DClk ,
                                            Reset => Reset ,
                                            Neg => DisplayNegParam ,
                                            WA => DisplayWAParam ,
                                            Tetr0 => BCD0 ,
                                            Tetr1 => BCD1 ,
                                            Tetr2 => BCD2 ,
                                            Tetr3 => BCD3 ,
                                            Dot => DotControl ,
                                            Seg_A => Seg_A ,
                                            Seg_B => Seg_B ,
                                            Seg_C => Seg_C ,
                                            Seg_D => Seg_D ,
                                            Seg_E => Seg_E ,
                                            Seg_F => Seg_F ,
                                            Seg_G => Seg_G ,
                                            Seg_K => Seg_K ,
                                            DSel0 => DSel0 ,
                                            DSel1 => DSel1 ,
                                            DSel2 => DSel2 ,
                                            DSel3 => DSel3 ) ;

  AutomatInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      DispMode <= '0' ;
      MachineState <= IdleState ;
      Context <= "00" ;
    else
      if Clk'event and Clk = '0' then
        case MachineState is
          when IdleState =>
            if ModeKeyStatus = '1' then
              MachineState <= DispModeOptionState ;
            elsif SetupKeyStatus = '1' then
              MachineState <= SetupOptionState ;
            else
              MachineState <= IdleState ;
            end if ;
          when SetupOptionState =>
            if Context = "00" then
              DispMode <= '1' ;
              Context <= "01" ;
            elsif Context = "01" then
              Context <= "10" ;
            elsif Context = "10" then
              Context <= "11" ;
            elsif Context = "11" then
              Context <= "00" ;
            end if ;
            MachineState <= EndOptionState ;
          when DispModeOptionState =>
            if Context = "00" then
              DispMode <= not DispMode ;
            end if ;
            MachineState <= EndOptionState ;
          when EndOptionState =>
            if ModeKeyStatus = '0' and SetupKeyStatus = '0' then
              MachineState <= IdleState ;
            else
              MachineState <= EndOptionState ;
            end if ;
        end case ;
      end if ;
    end if ;
  end process ;

end Behavioral ;
