--
-- **************************************************************************
-- **************************************************************************
-- **                                                                      **
-- **                      "Ukady CPLD w praktyce"                        **
-- **                                                                      **
-- **            Module: ARepKSrv (przydatny komponent)                    **
-- **            Module: Mux4DigDisp (przydatny komponent)                 **
-- **            Target device: XILINX CPLD                                **
-- **            Tool versions: ISE WebPACK 9.2i                           **
-- **            Date: BIAYSTOK, stycze 2009                             **
-- **                                                                      **
-- **                     Napisa : Andrzej Pawluczuk                      **
-- **                                                                      **
-- **************************************************************************
-- **************************************************************************
--
-----------------------------------------------------------------------------
library IEEE ;
use IEEE.STD_LOGIC_1164.ALL ;
use IEEE.STD_LOGIC_ARITH.ALL ;
use IEEE.STD_LOGIC_UNSIGNED.ALL ;

entity ARepKSrv is port ( Clk : in std_logic ;
                          TClk : in std_logic ;
                          Reset : in std_logic ;
                          KeyIn : in std_logic ;
                          Status : out std_logic ) ;
end ARepKSrv ;

architecture Behavioral of ARepKSrv is

  type AutomatStateType is ( IdleState ,
                             StartTimeState ,
                             WaitTimeState ,
                             KeyReady1State ,
                             KeyReady2State ,
                             StartWaitTimeFreeKeyState ,
                             WaitTimeFreeKeyState ,
                             StartAutoRepKeyState ,
                             AutoRepKeyState ) ;

  constant StandardDelay : std_logic_vector ( 4 downto 0 ) := "00011" ;
  constant StartRepeatDelay : std_logic_vector ( 4 downto 0 ) := "11100" ;
  constant RepeatDelay : std_logic_vector ( 4 downto 0 ) := "00100" ;

  signal AutomatState : AutomatStateType ;
  signal DelayCnt : std_logic_vector ( 4 downto 0 ) ;
  signal DelayCntEnable : std_logic ;
  signal DelayCntClear : std_logic ;

begin
  DelatCntInstance : process ( TClk , Reset , DelayCntClear , DelayCntEnable )
  begin
    if Reset = '0' or DelayCntClear = '0' then
      DelayCnt <= ( others => '0' ) ;
    else
      if DelayCntEnable = '1' then
        if TClk'event and TClk = '1' then
          DelayCnt <= DelayCnt + 1 ;
        end if ;
      end if ;
    end if ;
  end process ;

  AutomInstance : process ( Clk , Reset )
  begin
    if Reset = '0' then
      AutomatState <= IdleState ;
    else
      if Clk'event and Clk = '1' then
        case AutomatState is
          when IdleState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '1' ;
            Status <= '0' ;
            if KeyIn = '1' then
              AutomatState <= IdleState ;
            else
              AutomatState <= StartTimeState ;
            end if ;
          when StartTimeState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '0' ;
            Status <= '0' ;
            AutomatState <= WaitTimeState ;
          when WaitTimeState =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            Status <= '0' ;
            if KeyIn = '1' then
              AutomatState <= IdleState ;
            else
              if DelayCnt = StandardDelay then
                AutomatState <= KeyReady1State ;
              else
                AutomatState <= WaitTimeState ;
              end if ;
            end if ;
          when KeyReady1State =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            Status <= '1' ;
            AutomatState <= KeyReady2State ;
          when KeyReady2State =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            Status <= '0' ;
            if KeyIn = '1' then
              AutomatState <= StartWaitTimeFreeKeyState ;
            else
              if DelayCnt = StartRepeatDelay then
                AutomatState <= StartAutoRepKeyState ;
              else
                AutomatState <= KeyReady2State ;
              end if ;
            end if ;
          when StartWaitTimeFreeKeyState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '0' ;
            Status <= '0' ;
            AutomatState <= WaitTimeFreeKeyState ;
          when WaitTimeFreeKeyState =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            Status <= '0' ;
            if DelayCnt = StandardDelay then
              AutomatState <= IdleState ;
            else
              AutomatState <= WaitTimeFreeKeyState ;
            end if ;
          when StartAutoRepKeyState =>
            DelayCntEnable <= '0' ;
            DelayCntClear <= '0' ;
            Status <= '1' ;
            AutomatState <= AutoRepKeyState ;
          when AutoRepKeyState =>
            DelayCntEnable <= '1' ;
            DelayCntClear <= '1' ;
            Status <= '0' ;
            if KeyIn = '1' then
              AutomatState <= StartWaitTimeFreeKeyState ;
            else
              if DelayCnt = RepeatDelay then
                AutomatState <= StartAutoRepKeyState ;
              else
                AutomatState <= AutoRepKeyState ;
              end if ;
            end if ;
        end case ;
      end if ;
    end if ;
  end process ;

end Behavioral ;
